unit SrvProcess;

interface
uses Srv_Cli, WSocket, WSocketS, sysutils, Classes, db, dbTables;

type
  TServerConnBuffer = class;
  TSrvClient = class(TWSocketClient)
  public
    Buffer : TServerConnBuffer;
    constructor Create(AOwner:TComponent); override;
    destructor  Destroy; override;
  end;
  TServerConnBuffer = class(TConnBuffer)
  protected
    FQuery: TQuery;
    procedure ProcessData; override;
  public
    SessionName, DatabaseName : String;
    constructor Create( Socket: TWSocket ); override;
    destructor  Destroy; override;
    procedure   Log(const Msg:String); override;
    procedure do_cmdSQL;
    procedure do_cmdLoadFile;
    procedure do_cmdWho;
    procedure do_cmdError(const Msg: String);
    procedure do_cmdLogin;
  end;

var
  ServerLog:TStrings=nil;

implementation

constructor TSrvClient.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  Buffer:=TServerConnBuffer.Create( self );
end;

destructor TSrvClient.Destroy;
begin
  Buffer.Free;
  inherited Destroy;
end;

constructor TServerConnBuffer.Create( Socket: TWSocket );
begin
  inherited Create( Socket );
  FQuery := TQuery.Create(nil);
end;

destructor TServerConnBuffer.Destroy;
begin
  FQuery.Free;
  inherited Destroy;
end;

procedure TServerConnBuffer.Log(const Msg:String);
begin
  if ServerLog<>nil then
    ServerLog.add( Msg );
end;

procedure TServerConnBuffer.ProcessData;
begin

  Log ('Command: '+CommandNames[Command].Name );

  case Command of
    cmdLogout: begin
      FSocket.CloseDelayed;
    end;
    cmdLogin: begin
      do_cmdLogin;
    end;
    cmdTime: begin
      WriteParam( DateTimeToStr(Now) );
      Send;
    end;
    cmdSQL: begin
      do_cmdSQL;
    end;
    cmdLoadFile: begin
      do_cmdLoadFile;
    end;
    cmdWho: begin
      do_cmdWho;
    end
    else begin
      do_cmdError('Unknown command');
    end;
  end;
end;

procedure TServerConnBuffer.do_cmdSQL;
var
  SQLCommand:TSQLCommand;
  S1:String;
  I, RecCont: Integer;
begin
  try

    SQLCommand := TSQLCommand(ReadByte);

    FQuery.SQL.Text     := ReadParam; // SQL Text
    FQuery.SessionName  := SessionName;
    FQuery.DataBaseName := DataBaseName;

    while ReadByte=0 do begin // Tem parametros
      S1 := ReadParam;  // SQL Params
      with FQuery.ParamByName( StrDivide( S1, ';') ) do begin
        DataType := TFieldType( StrToInt( S1 ) );
        Value := ReadParam;
      end;
    end;

    if SQLCommand=cmdSQLExec then begin
      FQuery.ExecSQL;
      WriteByte( Byte( SQLCommand )); 
      WriteInt( FQuery.RowsAffected );
      FQuery.SQL.Clear; // Disconnect

      Send;
    end
    else begin
      FQuery.Open;

      WriteByte( Byte( SQLCommand ));
      WriteInt( FQuery.FieldCount );

      for I:=0 to FQuery.FieldCount-1 do begin
        WriteParam(
          FQuery.Fields[I].FieldName + ';'+
          IntToStr(Integer(FQuery.Fields[I].Datatype)) + ';'+
          IntToStr(FQuery.Fields[I].Size) + ';'+
          IntToStr(Integer(FQuery.Fields[I].Required)) );
      end;

      RecCont := 0;
      if SQLCommand=cmdSQLOpen then begin
        while not FQuery.Eof do Begin
          WriteByte(0); // Registro
          for I:=0 to FQuery.FieldCount-1 do begin
            WriteParam( FQuery.Fields[I].asString );
          end;
          Inc( RecCont );
          FQuery.Next;
        end;
      end;

      WriteByte(1); // Trailer
      WriteInt( RecCont );
      FQuery.SQL.Clear; // Disconnect

      Send;
    end;

  except
    on E:Exception do begin
      do_cmdError(E.ClassName+': '+E.Message);
    end;
  end
end;

procedure TServerConnBuffer.do_cmdLoadFile;
begin
end;

procedure TServerConnBuffer.do_cmdWho;
var
  I       : Integer;
  AClient : TSrvClient;
begin
end;

procedure TServerConnBuffer.do_cmdError(const Msg:String);
begin
  SetCommand( cmdError, Msg);
  Send;
end;

procedure TServerConnBuffer.do_cmdLogin();
begin
end;


end.
 